/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.servicemix.drools.model;

import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.NamespaceContext;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.expression.JAXPBooleanXPathExpression;
import org.apache.servicemix.expression.JAXPStringXPathExpression;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.w3c.dom.Element;

public class Message {
	private static final Log log = LogFactory.getLog(Message.class); 
	
    private static final SourceTransformer TRANFORMER = new SourceTransformer();
    
    private final NormalizedMessage message;
    private final NamespaceContext namespaceContext;
    
    public Message(NormalizedMessage message, NamespaceContext namespaceContext) {
        this.message = message;
        // Make sure message is re-readable
        this.namespaceContext = namespaceContext;
        Source content = message.getContent();
        if (content != null) {
            try {
                content = new DOMSource(TRANFORMER.toDOMElement(content));
                message.setContent(content);
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        
    }
    
    public NormalizedMessage getInternalMessage() {
        return this.message;
    }
    
    public boolean xpath(String xpath) throws Exception {
    	log.debug("xpath -> Evaluating ["+xpath+"]");
    	JAXPBooleanXPathExpression expression = new JAXPBooleanXPathExpression(xpath);
        if (this.namespaceContext != null) {
            expression.setNamespaceContext(this.namespaceContext);
        }else{
        	System.out.println(" ******** NS Context is null");
        }
        Boolean b = (Boolean) expression.evaluate(null, message);
        log.debug("xpath -> Evaluated ["+b.booleanValue()+"]");
        return b.booleanValue();
  
    }
    
    public String valueOf(String xpath) throws Exception {
    	log.debug("valueOf -> Evaluating ["+xpath+"]");
    	JAXPStringXPathExpression expression = new JAXPStringXPathExpression(xpath);
        if (this.namespaceContext != null) {
            expression.setNamespaceContext(this.namespaceContext);
        }else{
        	System.out.println(" ******** NS Context is null");
        }
        String result = (String)expression.evaluate(null, message);
        log.debug("valueOf -> Evaluated ["+result+"]");
        return result;
    }
    
    
    public Object getProperty(String name) {
        return message.getProperty(name);
    }
    
    public void setProperty(String name, Object value) {
        message.setProperty(name, value);
    }
    
    public Element getContent() {
        return (Element) ((DOMSource) message.getContent()).getNode();
    }
    /*
    public String dom4jValueOf(String xPathString) throws Exception{
    	try{
    		//System.out.println(" ** Drools eval** :: dom4jValueOf ["+xPathString+"] \n");
    		SourceTransformer st = new SourceTransformer();
    		String aString = st.toString(message.getContent());
    		System.out.println(aString);
    		Document doc =DocumentHelper.parseText(aString);
    		DefaultXPath xPath = new DefaultXPath(xPathString);
    		Map<String,String> nsMap = ((NamespaceContextImpl)namespaceContext).getNamespaces();
    		
    		xPath.setNamespaceURIs(nsMap);
    		String str = xPath.valueOf(doc);
    		//System.out.println(" ** Drools eval** :: Result is "+str);
    		return str;
    	}catch (Exception e) {
			e.printStackTrace();
			throw e;
    	}
    }
    */
    
    /*
    public boolean dom4jBooleanValueOf(String xPathString) throws Exception{
    	try{
    		//System.out.println(" ** Drools eval** :: dom4jBooleanValueOf ["+xPathString+"] \n");
    		SourceTransformer st = new SourceTransformer();
    		String aString = st.toString(message.getContent());
    		//System.out.println(" ** Drools eval** :: Message is  \n");
    		//System.out.println(aString);
    		Document doc =DocumentHelper.parseText(aString);
    		DefaultXPath xPath = new DefaultXPath(xPathString);
    		Map<String,String> nsMap = ((NamespaceContextImpl)namespaceContext).getNamespaces();
    		/*System.out.println("*** NamespaceContext ****");
    		for (String k : nsMap.keySet()){
    			System.out.println("["+k+"] -> ["+nsMap.get(k)+"]");
    		}
    		
    		xPath.setNamespaceURIs(nsMap);
    		boolean b = xPath.booleanValueOf(doc);
    		//System.out.println(" ** Drools eval** :: Result is "+b);
    		return b;
    	}catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
    }
    */
    
}
